"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof2 = require("babel-runtime/helpers/typeof");

var _typeof3 = _interopRequireDefault(_typeof2);

var _defineProperty2 = require("babel-runtime/helpers/defineProperty");

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _values = require("babel-runtime/core-js/object/values");

var _values2 = _interopRequireDefault(_values);

var _extends3 = require("babel-runtime/helpers/extends");

var _extends4 = _interopRequireDefault(_extends3);

var _objectWithoutProperties2 = require("babel-runtime/helpers/objectWithoutProperties");

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

var _toConsumableArray2 = require("babel-runtime/helpers/toConsumableArray");

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _keys = require("babel-runtime/core-js/object/keys");

var _keys2 = _interopRequireDefault(_keys);

var _set = require("babel-runtime/core-js/set");

var _set2 = _interopRequireDefault(_set);

exports.default = function (_ref) {
  var t = _ref.types;

  var name = "babel-plugin-captains-log";
  var callExpressions = new _set2.default();
  var evaluatedExpressions = new _set2.default();
  return {
    name: name,
    visitor: {
      Identifier: function Identifier(path, _ref2) {
        var opts = _ref2.opts;

        if (!looksLike(path.node, { name: "console" })) {
          return;
        }
        var settings = buildSettings(opts || {});
        var parentCallExp = path.findParent(t.isCallExpression);
        if (isTrackingConsoleCallStatement(path, parentCallExp, settings)) {
          callExpressions.add(parentCallExp);
        }
      },

      Program: {
        exit: function exit(_, _ref3) {
          var file = _ref3.file,
              opts = _ref3.opts;

          var settings = buildSettings(opts || {});
          callExpressions.forEach(function (callExp) {
            if (!callExp || evaluatedExpressions.has(callExp)) {
              return;
            }
            var options = settings[getConsoleCallMethodName(callExp)];
            var args = callExp.node.arguments;

            if (options.injectVariableName) {
              args = injectVariableNames(args);
            }

            if (options.injectScope) {
              var scope = findCallScope(callExp);
              args = prependArguments(args, scope);
            }

            if (options.injectFileName) {
              var filename = void 0;
              if (file) {
                filename = file.opts.filename;
              }
              var start = callExp.node.loc.start;
              var lineCol = "(" + start.line + ":" + start.column + ")";
              args = prependArguments(args, "" + filename + lineCol);
            }
            callExp.node.arguments = args;
          });
        }
      }
    }
  };

  function getConsoleCallMethodName(callExpression) {
    return callExpression.get("callee.property").node.name;
  }

  function isTrackingConsoleCallStatement(path, parentCallExp, settings) {
    return parentCallExp && parentCallExp.node.callee === path.parent && (0, _keys2.default)(settings).includes(getConsoleCallMethodName(parentCallExp));
  }

  function injectVariableNames() {
    var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

    return args.reduce(function (acc, arg) {
      if (t.isIdentifier(arg)) {
        return [].concat((0, _toConsumableArray3.default)(acc), [t.stringLiteral(arg.name), arg]);
      }
      return [].concat((0, _toConsumableArray3.default)(acc), [arg]);
    }, []);
  }

  function buildSettings(opts) {
    var methods = opts.methods,
        flags = (0, _objectWithoutProperties3.default)(opts, ["methods"]);
    // output spreads the flags over each method
    // in the future this could be expanded to allow method level config

    return (methods || defaultMethods).reduce(function (acc, curr) {
      return (0, _extends4.default)({}, acc, (0, _defineProperty3.default)({}, curr, (0, _values2.default)(flags).length ? flags : defaultSettings));
    }, {});
  }

  function findCallScope(path) {
    var scope = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    var parentFunc = path.findParent(t.isFunction);
    if (parentFunc) {
      if (t.isFunctionDeclaration(parentFunc)) {
        return findCallScope(parentFunc, [parentFunc.node.id.name].concat((0, _toConsumableArray3.default)(scope)));
      }
      if (t.isFunctionExpression(parentFunc) || t.isArrowFunctionExpression(parentFunc)) {
        if (looksLike(parentFunc.parent, { type: "VariableDeclarator" })) {
          return findCallScope(parentFunc.parentPath, [parentFunc.parent.id.name].concat((0, _toConsumableArray3.default)(scope)));
        }
        if (looksLike(parentFunc.parent, { type: "AssignmentExpression" })) {
          return findCallScope(parentFunc.parentPath, [parentFunc.parent.left.name].concat((0, _toConsumableArray3.default)(scope)));
        }
        if (looksLike(parentFunc.parent, { type: "ObjectProperty" })) {
          return findCallScope(parentFunc.parentPath, [parentFunc.parent.key.name].concat((0, _toConsumableArray3.default)(scope)));
        }
      }
      if (t.isObjectMethod(parentFunc) || parentFunc.isClassMethod()) {
        return findCallScope(parentFunc.parentPath, [parentFunc.node.key.name].concat((0, _toConsumableArray3.default)(scope)));
      }
    }
    return scope.length ? scope.join(".") + ":" : "";
  }

  function prependArguments() {
    var args = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];
    var value = arguments[1];

    if (value) {
      return [t.stringLiteral(value)].concat((0, _toConsumableArray3.default)(args));
    }
    return args;
  }
};

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var defaultSettings = {
  injectScope: true,
  injectVariableName: true,
  injectFileName: true
};

var defaultMethods = ["debug", "error", "exception", "info", "log", "warn"];

function looksLike(a, b) {
  return a && b && (0, _keys2.default)(b).every(function (bKey) {
    var bVal = b[bKey];
    var aVal = a[bKey];
    if (typeof bVal === "function") {
      return bVal(aVal);
    }
    return isPrimitive(bVal) ? bVal === aVal : looksLike(aVal, bVal);
  });
}

function isPrimitive(val) {
  return val == null || /^[sbn]/.test(typeof val === "undefined" ? "undefined" : (0, _typeof3.default)(val));
}